@@ -84,11 +84,11 @@ def keygetter(
                 dct = dct[sub_field]
             elif hasattr(dct, sub_field):
                 dct = getattr(dct, sub_field)
-        return dct
     except Exception as e:
         traceback.print_stack()
         print(f"Above error was {e}")
-    return None
+        return None
+    return dct
 
 
 def parse_lookup(obj: Mapping[str, Any], path: str, lookup: str) -> Optional[Any]:
@@ -117,7 +117,8 @@ def parse_lookup(obj: Mapping[str, Any], path: str, lookup: str) -> Optional[Any
     """
     try:
         if isinstance(path, str) and isinstance(lookup, str) and path.endswith(lookup):
-            if field_name := path.rsplit(lookup)[0]:
+            field_name = path.rsplit(lookup)[0]
+            if field_name is not None:
                 return keygetter(obj, field_name)
     except Exception:
         traceback.print_stack()
@@ -173,7 +174,7 @@ def lookup_icontains(
     if isinstance(data, str):
         return rhs.lower() in data.lower()
     if isinstance(data, Mapping):
-        return rhs.lower() in [k.lower() for k in data.keys()]
+        return rhs.lower() in [k.lower() for k in data]
 
     return False
 
@@ -233,7 +234,6 @@ def lookup_in(
             return rhs in data
         # TODO: Add a deep Mappingionary matcher
         # if isinstance(rhs, Mapping) and isinstance(data, Mapping):
-        #     return rhs.items() not in data.items()
     except Exception:
         return False
     return False
@@ -255,7 +255,6 @@ def lookup_nin(
             return rhs not in data
         # TODO: Add a deep Mappingionary matcher
         # if isinstance(rhs, Mapping) and isinstance(data, Mapping):
-        #     return rhs.items() not in data.items()
     except Exception:
         return False
     return False
@@ -296,6 +295,16 @@ LOOKUP_NAME_MAP: Mapping[str, LookupProtocol] = {
 }
 
 
+class PKRequiredException(Exception):
+    def __init__(self, *args: object):
+        return super().__init__("items() require a pk_key exists")
+
+
+class OpNotFound(ValueError):
+    def __init__(self, op: str, *args: object):
+        return super().__init__(f"{op} not in LOOKUP_NAME_MAP")
+
+
 class QueryList(list[T]):
     """Filter list of object/dictionaries. For small, local datasets.
 
@@ -450,17 +459,13 @@ class QueryList(list[T]):
 
     def items(self) -> list[T]:
         if self.pk_key is None:
-            raise Exception("items() require a pk_key exists")
+            raise PKRequiredException()
         return [(getattr(item, self.pk_key), item) for item in self]
 
     def __eq__(
         self,
         other: object,
         # other: Union[
-        #     "QueryList[T]",
-        #     List[Mapping[str, str]],
-        #     List[Mapping[str, int]],
-        #     List[Mapping[str, Union[str, Mapping[str, Union[List[str], str]]]]],
         # ],
     ) -> bool:
         data = other
@@ -469,7 +474,7 @@ class QueryList(list[T]):
             return False
 
         if len(self) == len(data):
-            for (a, b) in zip(self, data):
+            for a, b in zip(self, data):
                 if isinstance(a, Mapping):
                     a_keys = a.keys()
                     if a.keys == b.keys():
@@ -492,7 +497,7 @@ class QueryList(list[T]):
                     lhs, op = path.rsplit("__", 1)
 
                     if op not in LOOKUP_NAME_MAP:
-                        raise ValueError(f"{op} not in LOOKUP_NAME_MAP")
+                        raise OpNotFound(op=op)
                 except ValueError:
                     lhs = path
                     op = "exact"
@@ -514,7 +519,7 @@ class QueryList(list[T]):
                 if isinstance(matcher, list):
                     return obj in matcher
                 else:
-                    return obj == matcher
+                    return bool(obj == matcher)
 
             _filter = val_match
         else:
