@@ -65,6 +65,11 @@ if TYPE_CHECKING:
 F = TypeVar("F", bound=Callable[..., Any])
 
 
+class SubprocessCheckOutputError(Exception):
+    def __init__(self, output: str, *args: object):
+        return super().__init__(f"output is not str or bytes: {output}")
+
+
 if sys.platform == "win32":
     _ENV: "TypeAlias" = Mapping[str, str]
 else:
@@ -298,7 +303,7 @@ class SubprocessCommand(SkipDefaultFieldsReprMixin):
                 args=args or self.args,
                 encoding=encoding,
                 errors=errors,
-                text=text,
+                text=text or None,
                 universal_newlines=universal_newlines,
                 **kwargs,
             ).__dict__,
@@ -429,7 +434,8 @@ class SubprocessCommand(SkipDefaultFieldsReprMixin):
         output = subprocess.check_output(input=input, **params)
         if isinstance(output, (bytes, str)):
             return output
-        raise Exception(f"output is not str or bytes: {output}")
+
+        raise SubprocessCheckOutputError(output=output)
 
     @overload
     def run(
@@ -575,7 +581,7 @@ class SubprocessCommand(SkipDefaultFieldsReprMixin):
                 self,
                 universal_newlines=universal_newlines,
                 errors=errors,
-                text=text,
+                text=text or None,
                 **kwargs,
             ).__dict__,
             check=check,
